package com.taboola.flutter;


import android.content.Context;

import io.flutter.Log;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * FIXME: Open Flutter issue, adding MethodChannels to packages (plugin inside plugin essentially)
 * In order for Android to recognize these imports I added the flutter.jar.
 *  This might cause issues working with the Flutter project itself, might have to comment out dependency before building.
 *
 *  Current patchy workflow: Don't forget to comment in/out flutter.jar in Build.Gradle.
 *  Source: https://github.com/flutter/flutter/issues/19830
 */
public class TaboolaSdkPlugin implements FlutterPlugin, MethodCallHandler {
    private static Taboola sTaboola;

    @Override
    public void onAttachedToEngine(FlutterPluginBinding flutterPluginBinding) {
        final MethodChannel channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "taboola_sdk");
        channel.setMethodCallHandler(new com.taboola.flutter.TaboolaSdkPlugin());

        if (sTaboola == null){
            sTaboola = new Taboola(flutterPluginBinding.getApplicationContext());
        }
    }

    // This static function is optional and equivalent to onAttachedToEngine. It supports the old
    // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
    // plugin registration via this function while apps migrate to use the new Android APIs
    // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
    //
    // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
    // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
    // depending on the user's project. onAttachedToEngine or registerWith must both be defined
    // in the same class.
    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "taboola_sdk");
        channel.setMethodCallHandler(new com.taboola.flutter.TaboolaSdkPlugin());

        if (sTaboola == null){
            sTaboola = new Taboola(registrar.context());
        }
    }

    @Override
    public void onMethodCall(MethodCall call, Result result) {
        if (sTaboola == null) {
            Log.d("TaboolaSdkPlugin", "onMethodCall | sTaboola is null, method called = " + call.method);
            return;
        }

        if (call.method.equals("isUserOptOut")) { //Advertising Id related
            result.success(sTaboola.isLimitedAdTrackingEnabled());
        } else if (call.method.equals("getAdIdOrIdfa")) {
            result.success(sTaboola.getAdvertisingId());
        } else {
            result.notImplemented();
        }
    }

    @Override
    public void onDetachedFromEngine(FlutterPluginBinding binding) {
    }
}