package com.taboola.flutter.utils;

import android.content.Context;
import android.content.SharedPreferences;
import android.util.Log;

public class SharedPrefUtil {
    private static final String TAG = SharedPrefUtil.class.getSimpleName();

    private static final String SHARED_PREFS_KEY = "com.taboola.android.SHARED_PREFERENCES_KEY";
    private static final String ADVERTISING_ID_SHARED_PREFS_KEY = "com.taboola.android.ADVERTISING_ID_SHARED_PREFERENCES_KEY";
    private static final String LIMITED_AD_TRACKING_SHARED_PREFS_KEY = "com.taboola.android.LIMITED_TRACKING_SHARED_PREFERENCES_KEY";


    public static String getCachedAdvertisingId(Context context) {
        String id = getString(context, ADVERTISING_ID_SHARED_PREFS_KEY, null);
        Log.d(TAG, "getCachedAdvertisingId :: id = " + id);
        return id;
    }

    public static void setCachedAdvertisingId(Context context, String idValue) {
        putString(context, ADVERTISING_ID_SHARED_PREFS_KEY, idValue);
    }

    //Get shared preferences instance
    private static SharedPreferences getPreferences(Context context) {
        return context.getApplicationContext().getSharedPreferences(SHARED_PREFS_KEY, Context.MODE_PRIVATE);
    }

    //Helper methods for get/put String/Int/Long values
    private static String getString(Context context, String key, String defValue) {
        if (context == null) {
            return defValue; //return default value in case we're unable to open preferences
        }
        SharedPreferences preferences = getPreferences(context);
        return preferences.getString(key, defValue);
    }

    private static void putString(Context context, String key, String value) {
        if (context == null) {
            return;
        }
        SharedPreferences preferences = getPreferences(context);
        preferences.edit().putString(key, value).apply();
    }

    private static int getInt(Context context, String key, int defValue) {
        if (context == null) {
            return defValue; //return default value in case we're unable to open preferences
        }
        SharedPreferences preferences = getPreferences(context);
        return preferences.getInt(key, defValue);
    }

    private static void putInt(Context context, String key, int value) {
        if (context == null) {
            return;
        }
        SharedPreferences preferences = getPreferences(context);
        preferences.edit().putInt(key, value).apply();
    }


    private static long getLong(Context context, String key, long defValue) {
        if (context == null) {
            return defValue; //return default value in case we're unable to open preferences
        }
        SharedPreferences preferences = getPreferences(context);
        return preferences.getLong(key, defValue);
    }

    private static void putLong(Context context, String key, long value) {
        if (context == null) {
            return;
        }
        SharedPreferences preferences = getPreferences(context);
        preferences.edit().putLong(key, value).apply();
    }

}
