import 'dart:convert';
import 'package:package_info/package_info.dart';
import 'package:taboola_sdk/utils/text_utils.dart';
import 'package:taboola_sdk/utils/logger.dart';

class ApplicationDescriptor {
  static String _appName = ""; //publisher's app name
  static String _appv = ""; //publisher's app version
  static String _app = ""; //packageName(Android) / bundleId(iOS)


  ///NOTE: In our testing app, iOS will not return a name. This is because Flutter has an open issue with app names for iOS (which can be resolved manually).
  /// Our SDK is expected to deliver correct app names for publishers, if they solved their own name issue.
  ///
  /// Source: https://github.com/flutter/flutter/issues/43877
  static Future<String> getAppName() async {
    if (TextUtils.isEmptyOrNull(_appName)) {
      PackageInfo packageInfo = await PackageInfo.fromPlatform();
      if (packageInfo == null) {
        Logger.log('ApplicationDescriptor | getAppName() | packageInfo is null.');
        return "";
      }
      _appName = packageInfo.appName;

      try {
        _appName = HtmlEscape().convert(_appName);
      } catch (e) {
        Logger.logException('ApplicationDescriptor | HtmlEscape.convert() exception: ${e.toString()}');
      }
    }

    return _appName;
  }

  static Future<String> getAppVersion() async {
    if (TextUtils.isEmptyOrNull(_appv)) {
      PackageInfo packageInfo = await PackageInfo.fromPlatform();
      if (packageInfo == null) {
        Logger.log('ApplicationDescriptor | getAppName() | packageInfo is null.');
        return "";
      }
      _appv = packageInfo.version;
    }

    return _appv;
  }

  static Future<String> getAppPackageName() async {
    if (TextUtils.isEmptyOrNull(_app)) {
      PackageInfo packageInfo = await PackageInfo.fromPlatform();
      if (packageInfo == null) {
        Logger.log('ApplicationDescriptor | getAppName() | packageInfo is null.');
        return "";
      }

      _app = packageInfo.packageName;
    }
    return _app;
  }

//  /// Source: https://www.dartcn.com/articles/server/reflection-with-mirrors (info, deprecated from dart 2.0)
//  /// Source: https://pub.dev/packages/reflectable
//  static bool isAndroidXPresent() {
//    const String androidXPacakge1 = 'androidx.appcompat.app.AppCompatActivity';
//
//  }
//
//  static bool isKotlinPresent() {
//
//  }
}