import 'dart:collection';

import 'package:taboola_sdk/utils/logger.dart';

class ExtraPropertiesManager {
  static const String USER_OPT_OUT_KEY = "user_opt_out";

  HashMap<String,dynamic> _extraProperties;

  ExtraPropertiesManager(HashMap<String,dynamic> extraProperties){
    if (!isValidExtraPropertiesMap(extraProperties)){
      Logger.log("Taboola's ExtraProperties requires a <String, dynamic> map. It cannot accept type ${extraProperties.runtimeType}");
      //currently not stopping flow, just notifying user.
    }

    if (extraProperties == null) {
      _extraProperties = new HashMap<String,dynamic>();
    } else {
      _extraProperties = extraProperties;
    }
  }

  HashMap<String,dynamic> getExtraProperties() {
    return _extraProperties;
  }

  put(String key, dynamic value) {
    _extraProperties[key] = value;
  }

  putIfAbsent(String key, dynamic value) {
    _extraProperties.putIfAbsent(key, value);
  }

  /// This check is important as the 'dynamic' keywords allows external users to literally change our internal object types.
  /// A <String, String> map will pass compilation but limit our ExtraProperties map to String values only.
  bool isValidExtraPropertiesMap(HashMap<String, dynamic> extraProperties) {
    if (extraProperties == null) {
      return true; //it's ok, the map is optional for Taboola.init()
    }

    Type mapType = extraProperties.runtimeType;
    String mapTypeString = mapType.toString();

    return (mapTypeString.toLowerCase().contains("string") && mapTypeString.contains("dynamic"));
  }
}