import 'package:flutter/services.dart';
import 'package:taboola_sdk/utils/logger.dart';

/// This class is the Dart part of a 2-way bridge to native Android & iOS code.
///
/// The mirroring part is:
///   - In Android /taboola_sdk/android/src.main/java.com.taboola.flutter/TaboolaSdkPlugin.java
///     -- Editing this plugin is best done when editing /taboola_sdk/android/ in AndroidStudio and enabling a Gradle dependency on the Flutter.jar placed in libs.
///     -- Add your new method to the onMethodCall() filter function
///     -- After you finish editing in Android Studio, disable the Gradle dependency of Flutter.jar
///
///   - In iOS /taboola_sdk/ios/Classes/SwiftTaboolaSdkPlugin.swift
///     -- Add your new method to the handle() filter function
///
///Source: https://flutter.dev/docs/development/platform-integration/platform-channels
class TaboolaSdkPlugin {
  static const _nativeChannel = const MethodChannel('taboola_sdk');

  static Future<bool> isUserOptOut() async {
    if (_nativeChannel == null) {
      Logger.logException("TaboolaSdkPlugin | isUserOptOut() | _nativeChannel is null. Terminating.");
      return true; //lacking a value, better play it safe GDPR wise.
    }

    final bool isUserOptOut = await _nativeChannel.invokeMethod('isUserOptOut');
    Logger.log('TabooalaNative | isUserOptOut = $isUserOptOut');
    return isUserOptOut;
  }

  /// By definition, Android plugin will always return advId, iOS plugin will return an empty String if user has opted out.
  static Future<String> getAdIdOrIdfa() async {
    if (_nativeChannel == null) {
      Logger.logException("TaboolaSdkPlugin | getAdIdOrIdfa() | _nativeChannel is null. Terminating.");
      return "";
    }

    final String getAdIdOrIdfa = await _nativeChannel.invokeMethod('getAdIdOrIdfa');
    Logger.log('TabooalaNative | getAdIdOrIdfa = $getAdIdOrIdfa');
    return getAdIdOrIdfa;
  }

}