import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:webview_flutter/webview_flutter.dart';
import 'package:taboola_sdk/standard/webview_manager.dart';
import 'package:taboola_sdk/standard/webview_gesture_recognizer.dart';
import 'package:taboola_sdk/utils/logger.dart';
import 'dart:io' show Platform;


class TaboolaWebView extends StatelessWidget {
  String _initialUrl;
  Set<Factory<OneSequenceGestureRecognizer>> _savedFactoryRecognizers = Set<Factory<OneSequenceGestureRecognizer>>();
  WebViewGestureRecognizer _vertical;
  BuildContext _buildContext;

  final WebViewManager _webViewManager;

  final String _publisherId,_placement,_mode,_pageType,_pageUrl,_pageId,_targetType;

  TaboolaWebView(this._publisherId,this._placement,this._mode,this._pageType,this._pageUrl,this._pageId,this._targetType,this._webViewManager) {
    _initialUrl = "https://cdn.taboola.com/shared/templateJS.html?"
        "publisher=${_encodeParam(_publisherId)}"
        "&mode=${_encodeParam(_mode)}"
        "&pageType=${_encodeParam(_pageType)}"
        "&placement=${_encodeParam(_placement)}"
        "&targetType=${_encodeParam(_targetType)}"
        "&pageId=${_encodeParam(_pageId)}"
        "&pageUrl=${_encodeParam(_pageUrl)}"
        "&DIV_ID=taboola2"
        "&viewID=${DateTime.now().millisecondsSinceEpoch}";

    _webViewManager.taboolaWebViewDidScrollToTop = _taboolaWebViewDidScrollToTop;

    if (Platform.isAndroid) {
      //Only Android needs the "Scroll Squitch" solution. iOS Implements a stretching WebView without inner scrolling.
      _vertical = WebViewGestureRecognizer();//..onUpdate = (_) {};
      _vertical.scrollEnabled = false;

      _savedFactoryRecognizers = [
        Factory<WebViewGestureRecognizer>(
          () => _vertical,
        )
      ].toSet();
    }
  }

  /// An update event called from outside this WebView. This is where screen measurements decide if to enable our scroll switch logic.
  ///
  /// _vertical - This is the vertical GestureRecognizer we implement our ScrollSwitch logic in. It's a Taboola 'WebViewGestureRecognizer' object.
  void scrollListener(ScrollController controller, bool isFeed, bool canEnableScroll) {
    if (isFeed && Platform.isAndroid && canEnableScroll) {
      if (controller.offset >=controller.position.maxScrollExtent &&
          !controller.position.outOfRange) {
        Logger.log("reach the bottom");
        _vertical.scrollEnabled = true;
        _vertical.firstScrollEnabled = true;
      }
    }
    _sendNotifyExternalRects();
  }

  void _sendNotifyExternalRects() {
    if (_buildContext != null) {
      final RenderBox renderBox = _buildContext.findRenderObject();
      final position = renderBox.localToGlobal(Offset.zero);
      Size windowSize = MediaQuery.of(_buildContext).size;
      Size webViewSize = Size(windowSize.width, _buildContext.size.height);
      _webViewManager.webViewRect = position & webViewSize;
      _webViewManager.nativeWindowRect = Offset(0, 0) & windowSize;
      _webViewManager.notifyExternalRects();
    }

  }

  void _taboolaWebViewDidScrollToTop() {
    if (Platform.isAndroid) {
      _vertical.scrollEnabled = false;
      _vertical.firstScrollEnabled = true; //Set as the first scroll after scroll switch
      _vertical.yGlobal = 0; // Set y position to 0 because the webView scrolled to top
    }
  }

  @override
  Widget build(BuildContext context) {
    Logger.log(_initialUrl);
    _buildContext = context;
    return WebView(
      initialUrl: _initialUrl,
      javascriptMode: JavascriptMode.unrestricted,
      onWebViewCreated: (WebViewController controller) {
        _webViewManager.controller = controller;
        _sendNotifyExternalRects();
      },
      javascriptChannels: Set.from([_webViewManager.javascriptChannel()]),
      gestureRecognizers: _savedFactoryRecognizers,
      debuggingEnabled: true,
    );
  }

  String _encodeParam(String param) {
    if (param != null) {
      return Uri.encodeFull(param);
    }
    return "";
  }

}
