import 'package:flutter/cupertino.dart';
import 'package:flutter/gestures.dart';
import 'dart:core';

class WebViewGestureRecognizer extends OneSequenceGestureRecognizer {

  bool scrollEnabled = false;
  bool firstScrollEnabled = false;
  double yGlobal = 0;

  Offset _dragDistance = Offset.zero;

  WebViewGestureRecognizer();

  /// This is the only event that we managed to switch scrolling to web content
  @override
  void addPointer(PointerEvent event) {
    startTrackingPointer(event.pointer); //Allow handleEvent() to handle the logic.
  }

  @override
  String get debugDescription => 'customPan';

  @override
  void didStopTrackingLastPointer(int pointer) {}

  @override
  void handleEvent(PointerEvent event) {
    _dragDistance = _dragDistance + event.delta;
    if (event is PointerMoveEvent) {
      final double dy = _dragDistance.dy;
      double delta = dy - yGlobal;
      yGlobal= dy;
      final double dyAbs = dy.abs();
      final double dxAbs = _dragDistance.dx.abs();

      if (firstScrollEnabled) {
        if (delta > 0) {
          scrollEnabled = false;
        } else {
          scrollEnabled = true;
        }
      }
      firstScrollEnabled = false;

      bool acceptScroll = scrollEnabled && dyAbs > dxAbs && dyAbs > kTouchSlop; // kTouchSlop: "The distance a touch has to travel for the framework to be confident that the gesture is a scroll gesture". Source: constants.dart.

      if (acceptScroll) {
        // Vertical drag: accept
        resolve(GestureDisposition.accepted); // WebView internal will win
        _dragDistance = Offset.zero;
      } else if (dxAbs > kTouchSlop && dxAbs > dyAbs) {
        // Horizontal drag: stop tracking
        stopTrackingPointer(event.pointer); // Native external will win
        _dragDistance = Offset.zero;
      }
    }
  }



}